# Localisation Plugin

__author__  = 'SvaRoX'
__version__ = '0.2 - lvl 80'

import b3, b3.events, b3.plugin
import thread, threading, urllib, re
import math

#--------------------------------------------------------------------------------------------------
class LocalisationPlugin(b3.plugin.Plugin):
    _link = ''
    _countryRe = re.compile('Country: (.*)')
    _cityRe = re.compile('City: (.*)')
    _latRe = re.compile('Latitude: (.*)')
    _longRe = re.compile('Longitude: (.*)')
    _ccodeRe = re.compile('CountryCode: (.*)')
    # Format of the geoapi result page :
    # Country: France
    # City: Paris
    # Latitude: 48.8666992188
    # Longitude: 2.33330011368 

    def onLoadConfig(self):
        self._adminPlugin = self.console.getPlugin('admin')
        if not self._adminPlugin:
            self.error('Could not find admin plugin')
            return False

        self._adminPlugin.registerCommand(self, 'locate', 80, self.cmd_locate)
        self._adminPlugin.registerCommand(self, 'distance', 0, self.cmd_distance, 'dist')
        self._adminPlugin.registerCommand(self, 'friends', 0, self.cmd_friends)

        try:
            self._minLevel = self.config.getint('settings', 'minlevel')
        except:
            pass
        self.debug('Minimum Level to use commands = %d' % self._minLevel)

        try:
            self._link = self.config.get('settings', 'webaddress')
        except:
            self.error('Could not get geoip web address')
        self.debug('Geoip link = %s' % self._link)

        try:
            self._db_table = self.config.get('settings', 'db_table')
            self._db_id = self.config.get('settings', 'db_id')
            self._db_city = self.config.get('settings', 'db_city')
            self._db_country = self.config.get('settings', 'db_country')
            self._db_latitude = self.config.get('settings', 'db_latitude')
            self._db_longitude = self.config.get('settings', 'db_longitude')
            self._db_countrycode = self.config.get('settings', 'db_countrycode')
        except:
            self.error('Could not get database parameters')

        self.query = self.console.storage.query

        self.registerEvent(b3.events.EVT_CLIENT_CONNECT)

    def onEvent(self, event):
        if event.type == b3.events.EVT_CLIENT_CONNECT:
            thread.start_new_thread(self.updateLocation, (event.client,))
        
    def cmd_locate(self, data, client, cmd=None):
        """\
        <player> : locate a player (city and country)
        """
        if not data:
            client.message('Wrong parameter, try !help locate')
        else:
            m = self._adminPlugin.parseUserCmd(data)
            if m:
                sclient = self._adminPlugin.findClientPrompt(m[0], client)
                if not sclient:
                    pass
                    # cmd.sayLoudOrPM(client, 'No player found')
                else:
                    thread.start_new_thread(self.getLocation, (client, sclient))
                    # self.locate(sclient)

    def cmd_distance(self, data, client, cmd=None):
        """\
        <player> : displays how far you are from a player
        """
        if not data:
            client.message('Wrong parameter, try !help distance')
        else:
            m = self._adminPlugin.parseUserCmd(data)
            if m:
                sclient = self._adminPlugin.findClientPrompt(m[0], client)
                if not sclient:
                    pass
                    # cmd.sayLoudOrPM(client, 'No player found')
                else:
                    thread.start_new_thread(self.getDistance, (client, sclient))
        
    def cmd_friends(self, data, client, cmd=None):
        """\
        [<distance>] : displays who is connected from a circle of <distance> km radius
        """
        dist = 100.0
        if data:
            try:
                dist = float(data)
            except:
                client.message('Wrong parameter, try !help friends')
                return
        else:
            try:
                dist = self.config.getfloat('settings', 'radius')
            except:
                pass
        thread.start_new_thread(self.getFriends, (client, dist))

    def updateLocation(self, client):
        f = None
        cursor = None
        country = 'Country not found'
        city = 'City not found'
        lat = 'Latitude not found'
        long = 'Longitude not found'
        ccode = 'oO'

        try:
            # f = urllib.urlopen('http://localhost:8100/osef/geoip.php?ip=%s' % '88.171.40.117')
            f = urllib.urlopen(self._link % client.ip)
            # f = urllib.urlopen(self._link % '81.210.151.47')
            # f = urllib.urlopen(self._link % '188.155.108.212')
            # f = urllib.urlopen('http://www.dswp.de/old/dswp_tools/svarox/geoip.php?ip=%s' % '81.210.151.47')
            # f = urllib.urlopen(self._link % '138.246.7.235')
        except:
            self.error('Unable to retrieve localisation data from webapi')
            return

        countryMO = self._countryRe.match(f.readline())
        if countryMO != None:
            country = countryMO.group(1)

        cityMO = self._cityRe.match(f.readline())
        if cityMO != None:
            city = cityMO.group(1)

        latMO = self._latRe.match(f.readline())
        if latMO != None:
            lat = latMO.group(1)

        longMO = self._longRe.match(f.readline())
        if longMO != None:
            long = longMO.group(1)

        ccodeMO = self._ccodeRe.match(f.readline())
        if ccodeMO != None:
            ccode = ccodeMO.group(1)

        self.debug('ClientID %d: City = %s, Country = %s, Latitude = %s, Longitude = %s, Country code = %s' % (client.id, city, country, lat, long, ccode))

        q = 'SELECT * FROM %s WHERE %s = %s' % (self._db_table, self._db_id, client.id)
        try:
            cursor = self.query(q)
        except:
            self.error('Can\'t execute query : %s' % q)
        if (cursor and (cursor.rowcount > 0) ):
            r = cursor.getRow()
            self.debug('Client already exists in DB')
            if (country != r[self._db_country]) or (city != r[self._db_city]) or (lat != r[self._db_latitude]) or (long != r[self._db_longitude]) or (ccode != r[self._db_countrycode]):
                self.debug('Localisation data found for the client %s' % client.exactName)
                q = 'UPDATE %s SET %s=\'%s\', %s=\'%s\', %s=\'%s\', %s=\'%s\', %s=\'%s\' WHERE %s=%s' % (self._db_table, self._db_city, city, self._db_country, country, self._db_latitude, lat, self._db_longitude, long, self._db_countrycode, ccode, self._db_id, r[self._db_id])
                try:
                    self.query(q)
                except:
                    self.error('Can\'t execute query : %s' % q)
        else:
            self.debug('Client doesnt exist')
            q = 'INSERT INTO %s (%s, %s, %s, %s, %s, %s) VALUES (%s, \'%s\', \'%s\', \'%s\', \'%s\', \'%s\')' % (self._db_table, self._db_id, self._db_city, self._db_country, self._db_latitude, self._db_longitude, self._db_countrycode, client.id, city, country, lat, long, ccode)
            try:
                cursor = self.query(q)
            except:
                self.error('Can\'t execute query : %s' % q)
            if (cursor and cursor.rowcount > 0):
                lastid = cursor.lastrowid
                self.debug('Localisation data have been inserted for user %s' % lastid)
        client.setvar(self, 'city', city)
        client.setvar(self, 'country', country)
        client.setvar(self, 'latitude', lat)
        client.setvar(self, 'longitude', long)
        client.setvar(self, 'ccode', ccode)

    def getLocation(self, client, sclient):
        message = 'Cannot locate %s' % sclient.exactName

        if sclient.isvar(self, 'city') and sclient.isvar(self, 'country'):
            message = '^3%s is connected from ^1%s^7, ^2%s' % (sclient.exactName, sclient.var(self, 'city').value, sclient.var(self, 'country').value)
        client.message(message)


    def getDistance(self, client, sclient):
        milesFactor = 0.6213724
        message = 'Cannot calculte the distance between you and %s' % sclient.exactName

        if sclient.isvar(self, 'latitude') and sclient.isvar(self, 'longitude'):
            try:
                float(client.var(self, 'latitude').value)
                float(client.var(self, 'longitude').value)
                float(sclient.var(self, 'latitude').value)
                float(sclient.var(self, 'longitude').value)
            except:
                client.message(message)
                return
            distance = self.distanceCalc(float(client.var(self, 'latitude').value), float(client.var(self, 'longitude').value), float(sclient.var(self, 'latitude').value), float(sclient.var(self, 'longitude').value))
            message = '^1%s^7km/^2%s^7mi ^7away from ^3%s' % (int(distance), int(distance*milesFactor), sclient.exactName)
        client.message(message)

    def distanceCalc(self, lat1, long1, lat2, long2):
        R = 6378.0
        (x1, y1, x2, y2) = map(math.radians, (lat1, long1, lat2, long2))
        return R*math.acos(math.cos(x1)*math.cos(x2)*math.cos(y2-y1) + math.sin(x1)*math.sin(x2))

    def getFriends(self, client, dist):
        lat1 = 0.0
        long1 = 0.0
        if not client.isvar(self, 'latitude') or not client.isvar(self, 'longitude'):
            client.message('Cannot get friends list :-(')
            return
        else:
            try:
                lat1 = float(client.var(self, 'latitude').value)
                long1 = float(client.var(self, 'longitude').value)
            except:
                client.message('Cannot get friends list :-(')
                return

        message = ''
        friendsList = []
        clients = self.console.clients.getList()
        for c in clients:
            if c.cid != client.cid and c.isvar(self, 'latitude') and c.isvar(self, 'longitude'):
                try:
                    lat2 = float(c.var(self, 'latitude').value)
                    long2 = float(c.var(self, 'longitude').value)
                except:
                    continue
                    # pass
                if self.distanceCalc(lat1, long1, lat2, long2) <= dist:
                    friendsList.append(c.exactName)
        if len(friendsList) == 0:
            message = '^7No friend have been found in less than %s km away from you :-(' % str(dist)
        else:
            message = '^1%d ^7friend(s) have been found : ^2%s' % (len(friendsList), str.join("^7, ^2", friendsList))
        client.message(message)
